﻿using System;
using System.Data;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Telerik.JustMock;
using Telerik.OpenAccess.Data.Common;
using System.Data.Common;
using OpenAccessModel2;
using System.Collections.Generic;
using Telerik.JustMock.Helpers;

namespace UnitTestProject1
{
    [TestClass]
    public class UnitTest1
    {
        [TestMethod]
        public void TestAbstract()
        {
            var mock = Mock.Create<DbDataReader>();

            bool isTrue = false;

            Mock.Arrange(() => mock.HasRows).Returns(true);

            isTrue = mock.HasRows;

            Assert.IsTrue(isTrue);
        }

        [TestMethod]
        public void TestOAConnection()
        {
            var oaConnectionMock = Mock.Create<OAConnection>(Constructor.Mocked);
            var oaCommand = Mock.Create<OACommand>();
            oaCommand.CommandText = "test command";


            Mock.Arrange(() => oaConnectionMock.CreateCommand()).Returns(oaCommand);

            OACommand actualCommand = oaConnectionMock.CreateCommand();

            Assert.AreEqual(oaCommand.CommandText, actualCommand.CommandText);
        }

        [TestMethod]
        public void TestOACommand()
        {
            var oaCommandMock = Mock.Create<OACommand>();

            string expected = "Test";

            Mock.Arrange(() => oaCommandMock.CommandText).Returns(expected);

            var actual = oaCommandMock.CommandText;

            Assert.AreEqual(expected, actual);
        }

        [TestMethod]
        public void TestCnnCmdReader()
        {
            var dbContext = Mock.Create<EntitiesModel>();
            var oaConnectionMock = Mock.Create<OAConnection>(Constructor.Mocked);
            var oaCommand = Mock.Create<OACommand>();
            var oaReader = Mock.Create<OADataReader>();

            Mock.Arrange(() => dbContext.Connection).Returns(oaConnectionMock);
            Mock.Arrange(() => oaConnectionMock.Open()).DoNothing();
            Mock.Arrange(() => oaConnectionMock.CreateCommand()).Returns(oaCommand);
            Mock.Arrange(() => oaCommand.ExecuteReader()).Returns(oaReader);
            Mock.Arrange(() => oaReader.HasRows).Returns(true);
            Mock.Arrange(() => dbContext.Translate<Car>(oaReader)).ReturnsCollection(FakeCollection());
            Mock.Arrange(() => dbContext.AttachCopy<Car>(Arg.IsAny<Car>())).DoNothing().MustBeCalled();

            if (dbContext != null)
            {
                Car myCar = new Car();

                var x = AppDomain.CurrentDomain.GetAssemblies();
                Array.Sort(x, (a, b) => a.FullName.CompareTo(b.FullName));

                using (OAConnection connection = dbContext.Connection)
                {
                    using (OACommand command = connection.CreateCommand())
                    {
                        command.CommandType = System.Data.CommandType.StoredProcedure;
                        command.CommandText = "SpName";

                        using (DbDataReader dataReader = command.ExecuteReader())
                        {
                            if (dataReader.HasRows)
                            {
                                IEnumerable<Car> carName = dbContext.Translate<Car>(dataReader);

                                foreach (Car c in carName)
                                {
                                    myCar = dbContext.AttachCopy<Car>(c);
                                }
                            }
                        }
                    }
                }
            }

            Mock.Assert(() => dbContext);
        }

        public IEnumerable<Car> FakeCollection()
        {
            return new List<Car> { new Car() };
        }
    }
}
